'use strict';

const fs = require('node:fs/promises');
const fs$1 = require('node:fs');
const path = require('node:path');
const postcss = require('postcss');
const atImport = require('postcss-import');
const t = require('@babel/types');
const babelGenerate = require('@babel/generator');
const node_url = require('node:url');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

function _interopNamespaceCompat(e) {
  if (e && typeof e === 'object' && 'default' in e) return e;
  const n = Object.create(null);
  if (e) {
    for (const k in e) {
      n[k] = e[k];
    }
  }
  n.default = e;
  return n;
}

const fs__default = /*#__PURE__*/_interopDefaultCompat(fs);
const fs__default$1 = /*#__PURE__*/_interopDefaultCompat(fs$1);
const path__default = /*#__PURE__*/_interopDefaultCompat(path);
const postcss__default = /*#__PURE__*/_interopDefaultCompat(postcss);
const atImport__default = /*#__PURE__*/_interopDefaultCompat(atImport);
const t__namespace = /*#__PURE__*/_interopNamespaceCompat(t);
const babelGenerate__default = /*#__PURE__*/_interopDefaultCompat(babelGenerate);

const layerNodes = {
  base: null,
  components: null,
  utilities: null
  // variants: null
};
const markedLayerKey = "__tw_layer_name__";
const layerNodesKeys = Object.keys(layerNodes);

const markLayerPlugin = () => {
  return {
    postcssPlugin: "postcss-css-to-tailwindcss-plugin-mark-layer-plugin",
    Once(root) {
      root.walkAtRules("layer", (rule) => {
        if (layerNodesKeys.includes(rule.params)) {
          const layerName = rule.params;
          for (const node of rule.nodes) {
            Object.defineProperty(node, markedLayerKey, {
              value: layerName,
              enumerable: false,
              configurable: true
            });
          }
          rule.parent?.insertBefore(rule, rule.nodes);
          rule.remove();
        }
      });
    }
  };
};
markLayerPlugin.postcss = true;
const extractLayerPlugin = (options) => {
  const { ctx } = options;
  return {
    postcssPlugin: "postcss-css-to-tailwindcss-plugin-extract-layer-plugin",
    OnceExit(root) {
      root.walkRules((rule) => {
        const layerName = rule[markedLayerKey];
        if (layerName && layerNodesKeys.includes(layerName)) {
          ctx.append(layerName, rule);
        }
      });
    }
  };
};
extractLayerPlugin.postcss = true;

function tryLoadSass() {
  return import('sass');
}
function tryLoadSassSync() {
  return require("sass");
}
function isExtSassFile(extname) {
  return extname === ".scss" || extname === ".sass";
}
async function sassCompile(path, sassOptions) {
  const { compile } = await tryLoadSass();
  const { css } = compile(path, sassOptions);
  return css;
}
function sassCompileSync(path, sassOptions) {
  const { compile } = tryLoadSassSync();
  const { css } = compile(path, sassOptions);
  return css;
}

const expandAST = ["addBase", "addComponents", "addUtilities", "theme", "addVariant", "config", "corePlugins", "e", "matchComponents", "matchUtilities", "matchVariant"].map(
  (x) => {
    return t__namespace.objectProperty(t__namespace.identifier(x), t__namespace.identifier(x));
  }
);
function expandAPI() {
  return expandAST;
}
function unwrapThemeFunctionArg(str) {
  if (str) {
    const start = str[0];
    const last = str.at(-1);
    if (start === '"' && last === '"' || start === "'" && last === "'") {
      return str.slice(1, -1);
    }
  }
  return str;
}
function makeObjectProperty(key, value) {
  return t__namespace.objectProperty(t__namespace.stringLiteral(key), value);
}
function makeObjectExpression(nodes) {
  return nodes.reduce((acc, _cur) => {
    if (_cur.type === "rule") {
      const cur = _cur;
      acc.push(
        t__namespace.objectProperty(
          t__namespace.stringLiteral(cur.selector),
          t__namespace.objectExpression(
            cur.nodes.reduce((cacc, ccur) => {
              if (ccur.type === "decl") {
                const v = ccur.value;
                const arr = [...v.matchAll(/theme\(([^()]+?)\)/g)];
                if (arr.length > 0) {
                  const first = arr[0];
                  if (first[0] === v) {
                    cacc.push(makeObjectProperty(ccur.prop, t__namespace.callExpression(t__namespace.identifier("theme"), [t__namespace.stringLiteral(unwrapThemeFunctionArg(first[1]))])));
                  } else {
                    let p = 0;
                    const quasis = [];
                    const expressions = [];
                    for (let i = 0; i < arr.length; i++) {
                      const hit = arr[i];
                      if (typeof hit.index === "number") {
                        const raw = v.slice(p, hit.index);
                        p = hit.index + hit[0].length;
                        quasis.push(t__namespace.templateElement({ raw }));
                        expressions.push(t__namespace.callExpression(t__namespace.identifier("theme"), [t__namespace.stringLiteral(unwrapThemeFunctionArg(hit[1]))]));
                        if (i === arr.length - 1) {
                          quasis.push(t__namespace.templateElement({ raw: v.slice(p) }));
                        }
                      }
                    }
                    cacc.push(makeObjectProperty(ccur.prop, t__namespace.templateLiteral(quasis, expressions)));
                  }
                } else {
                  cacc.push(makeObjectProperty(ccur.prop, t__namespace.stringLiteral(v)));
                }
              }
              return cacc;
            }, [])
          )
        )
      );
    }
    return acc;
  }, []);
}
function createGenerator() {
  const callFnId = "_plugin";
  const pluginName = "css2TwPlugin";
  const requireStatement = t__namespace.variableDeclaration("const", [
    t__namespace.variableDeclarator(t__namespace.identifier(callFnId), t__namespace.callExpression(t__namespace.identifier("require"), [t__namespace.stringLiteral("tailwindcss/plugin")]))
  ]);
  const exportsStatement = t__namespace.expressionStatement(t__namespace.assignmentExpression("=", t__namespace.memberExpression(t__namespace.identifier("module"), t__namespace.identifier("exports")), t__namespace.identifier(pluginName)));
  function generate(ctx, opt) {
    const ast = t__namespace.file(
      t__namespace.program([
        requireStatement,
        t__namespace.variableDeclaration("const", [
          t__namespace.variableDeclarator(
            t__namespace.identifier(pluginName),
            t__namespace.callExpression(t__namespace.identifier(callFnId), [
              t__namespace.functionExpression(
                null,
                [t__namespace.objectPattern(expandAPI())],
                t__namespace.blockStatement([
                  // addBase
                  t__namespace.expressionStatement(t__namespace.callExpression(t__namespace.identifier("addBase"), [t__namespace.objectExpression(makeObjectExpression(ctx.getNodes("base")))])),
                  // addComponents
                  t__namespace.expressionStatement(t__namespace.callExpression(t__namespace.identifier("addComponents"), [t__namespace.objectExpression(makeObjectExpression(ctx.getNodes("components")))])),
                  // addUtilities
                  t__namespace.expressionStatement(t__namespace.callExpression(t__namespace.identifier("addUtilities"), [t__namespace.objectExpression(makeObjectExpression(ctx.getNodes("utilities")))]))
                ])
              )
            ])
          )
        ]),
        exportsStatement
      ])
    );
    const res = babelGenerate__default(ast, opt);
    return res.code;
  }
  return {
    generate
  };
}

function isObject(value) {
  return value !== null && typeof value === "object";
}
function _defu(baseObject, defaults, namespace = ".", merger) {
  if (!isObject(defaults)) {
    return _defu(baseObject, {}, namespace, merger);
  }
  const object = Object.assign({}, defaults);
  for (const key in baseObject) {
    if (key === "__proto__" || key === "constructor") {
      continue;
    }
    const value = baseObject[key];
    if (value === null || value === void 0) {
      continue;
    }
    if (merger && merger(object, key, value, namespace)) {
      continue;
    }
    if (Array.isArray(value) && Array.isArray(object[key])) {
      object[key] = [...value, ...object[key]];
    } else if (isObject(value) && isObject(object[key])) {
      object[key] = _defu(
        value,
        object[key],
        (namespace ? `${namespace}.` : "") + key.toString(),
        merger
      );
    } else {
      object[key] = value;
    }
  }
  return object;
}
function createDefu(merger) {
  return (...arguments_) => (
    // eslint-disable-next-line unicorn/no-array-reduce
    arguments_.reduce((p, c) => _defu(p, c, "", merger), {})
  );
}
const defu = createDefu();

function getDefaults() {
  const pkgPath = require.resolve("tailwindcss/package.json");
  const nodeModulesPath = path__default.dirname(path__default.dirname(pkgPath));
  return {
    atImportOptions: {},
    sassOptions: {
      importers: [
        {
          // https://github.com/webpack-contrib/sass-loader/blob/master/src/index.js#L55
          findFileUrl(url) {
            const twUrl = node_url.pathToFileURL(path__default.resolve(nodeModulesPath, url));
            return new URL(twUrl);
          }
        }
      ]
    },
    tailwindcssConfig: "",
    generatorOptions: {}
  };
}
function getOptions(opts) {
  return defu(opts, getDefaults());
}

function createContext(opts) {
  const options = getOptions(opts);
  const layersMap = /* @__PURE__ */ new Map();
  function resetLayersMap() {
    layersMap.set("base", []);
    layersMap.set("components", []);
    layersMap.set("utilities", []);
  }
  resetLayersMap();
  function getNodes(key) {
    const arrRef = layersMap.get(key);
    if (Array.isArray(arrRef)) {
      return arrRef;
    }
    const arr = [];
    layersMap.set(key, arr);
    return arr;
  }
  function append(key, nodes) {
    const arr = getNodes(key);
    if (Array.isArray(nodes)) {
      arr.push(...nodes);
    } else {
      arr.push(nodes);
    }
  }
  const generator = createGenerator();
  return {
    layersMap,
    append,
    getNodes,
    getPluginsSync() {
      const { tailwindcssConfig } = options;
      const opt = { ctx: this };
      const plugins = [markLayerPlugin()];
      if (tailwindcssConfig) {
        const tailwindcss = require("tailwindcss");
        plugins.push(tailwindcss(tailwindcssConfig));
      }
      plugins.push(extractLayerPlugin(opt));
      return plugins;
    },
    async getPlugins() {
      const { tailwindcssConfig } = options;
      const opt = { ctx: this };
      const plugins = [markLayerPlugin()];
      if (tailwindcssConfig) {
        const { default: tailwindcss } = await import('tailwindcss');
        plugins.push(tailwindcss(tailwindcssConfig));
      }
      plugins.push(extractLayerPlugin(opt));
      return plugins;
    },
    async process(entry) {
      const { sassOptions, atImportOptions } = options;
      const plugins = await this.getPlugins();
      let css;
      const extname = path__default.extname(entry);
      if (isExtSassFile(extname)) {
        try {
          css = await sassCompile(entry, sassOptions);
        } catch {
          throw new Error(`file: ${path__default} is skipped. please confirm you have installed \`sass\`!`);
        }
      } else {
        css = await fs__default.readFile(entry, "utf8");
        atImportOptions.root = path__default.dirname(entry);
        plugins.unshift(atImport__default(atImportOptions));
      }
      const res = await postcss__default(plugins).process(css, {
        from: void 0
      });
      return res;
    },
    processSync(entry) {
      const { sassOptions, atImportOptions } = options;
      const plugins = this.getPluginsSync();
      let css;
      const extname = path__default.extname(entry);
      if (isExtSassFile(extname)) {
        try {
          css = sassCompileSync(entry, sassOptions);
        } catch {
          throw new Error(`file: ${path__default} is skipped. please confirm you have installed \`sass\`!`);
        }
      } else {
        css = fs__default$1.readFileSync(entry, "utf8");
        atImportOptions.root = path__default.dirname(entry);
      }
      const res = postcss__default(plugins).process(css).sync();
      return res;
    },
    generate: function() {
      const { generatorOptions } = options;
      return generator.generate(this, generatorOptions);
    }
  };
}

exports.createContext = createContext;
