'use strict';

const path = require('node:path');
const fs = require('node:fs');
const plugin = require('tailwindcss/plugin');
const md5 = require('md5');
const index = require('./index.cjs');
const utils = require('./shared/css-to-tailwindcss-plugin.68c8502e.cjs');
require('node:fs/promises');
require('postcss');
require('postcss-import');
require('@babel/types');
require('@babel/generator');
require('node:url');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

const path__default = /*#__PURE__*/_interopDefaultCompat(path);
const fs__default = /*#__PURE__*/_interopDefaultCompat(fs);
const plugin__default = /*#__PURE__*/_interopDefaultCompat(plugin);
const md5__default = /*#__PURE__*/_interopDefaultCompat(md5);

function generateTempPlugin(entry, p) {
  const ctx = index.createContext();
  ctx.processSync(entry);
  const code = ctx.generate();
  fs__default.writeFileSync(p, code, "utf8");
  return code;
}
function getDefaultCacheDir() {
  return path__default.resolve(process.cwd(), "node_modules", ".css-to-tailwindcss-plugin");
}
const tailwindcss = plugin__default.withOptions(
  (options) => {
    const cacheDir = options.cacheDir ?? getDefaultCacheDir();
    utils.ensureDir(cacheDir);
    const indexFilePath = path__default.resolve(cacheDir, "index.json");
    let hashMap = {};
    function loadCache() {
      if (fs__default.existsSync(indexFilePath)) {
        try {
          hashMap = JSON.parse(fs__default.readFileSync(indexFilePath, "utf8"));
        } catch {
          console.log(`parse json: ${indexFilePath} failed!`);
        }
      }
    }
    function writeCacheIndexFile(data) {
      fs__default.writeFileSync(indexFilePath, JSON.stringify(data), "utf8");
    }
    loadCache();
    return function(api) {
      const targetPlugins = [];
      for (const entry of options.entries) {
        const fileHash = md5__default(entry);
        const cssHash = md5__default(fs__default.readFileSync(entry));
        const p = path__default.resolve(cacheDir, fileHash) + ".js";
        if (fs__default.existsSync(p)) {
          if (hashMap[fileHash] !== cssHash) {
            generateTempPlugin(entry, p);
            hashMap[fileHash] = cssHash;
          }
        } else {
          generateTempPlugin(entry, p);
          hashMap[fileHash] = cssHash;
        }
        targetPlugins.push(p);
      }
      for (const tmpPlugin of targetPlugins) {
        const fn = require(tmpPlugin);
        if (typeof fn === "function") {
          fn(api);
        }
      }
      writeCacheIndexFile(hashMap);
    };
  },
  () => {
    return {};
  }
);

module.exports = tailwindcss;
