import path from 'node:path';
import fs from 'node:fs';
import plugin from 'tailwindcss/plugin';
import md5 from 'md5';
import { createContext } from './index.mjs';
import { e as ensureDir } from './shared/css-to-tailwindcss-plugin.10c8b9ca.mjs';
import 'node:fs/promises';
import 'postcss';
import 'postcss-import';
import '@babel/types';
import '@babel/generator';
import 'node:url';



// -- Unbuild CommonJS Shims --
import __cjs_url__ from 'url';
import __cjs_path__ from 'path';
import __cjs_mod__ from 'module';
const __filename = __cjs_url__.fileURLToPath(import.meta.url);
const __dirname = __cjs_path__.dirname(__filename);
const require = __cjs_mod__.createRequire(import.meta.url);
function generateTempPlugin(entry, p) {
  const ctx = createContext();
  ctx.processSync(entry);
  const code = ctx.generate();
  fs.writeFileSync(p, code, "utf8");
  return code;
}
function getDefaultCacheDir() {
  return path.resolve(process.cwd(), "node_modules", ".css-to-tailwindcss-plugin");
}
const tailwindcss = plugin.withOptions(
  (options) => {
    const cacheDir = options.cacheDir ?? getDefaultCacheDir();
    ensureDir(cacheDir);
    const indexFilePath = path.resolve(cacheDir, "index.json");
    let hashMap = {};
    function loadCache() {
      if (fs.existsSync(indexFilePath)) {
        try {
          hashMap = JSON.parse(fs.readFileSync(indexFilePath, "utf8"));
        } catch {
          console.log(`parse json: ${indexFilePath} failed!`);
        }
      }
    }
    function writeCacheIndexFile(data) {
      fs.writeFileSync(indexFilePath, JSON.stringify(data), "utf8");
    }
    loadCache();
    return function(api) {
      const targetPlugins = [];
      for (const entry of options.entries) {
        const fileHash = md5(entry);
        const cssHash = md5(fs.readFileSync(entry));
        const p = path.resolve(cacheDir, fileHash) + ".js";
        if (fs.existsSync(p)) {
          if (hashMap[fileHash] !== cssHash) {
            generateTempPlugin(entry, p);
            hashMap[fileHash] = cssHash;
          }
        } else {
          generateTempPlugin(entry, p);
          hashMap[fileHash] = cssHash;
        }
        targetPlugins.push(p);
      }
      for (const tmpPlugin of targetPlugins) {
        const fn = require(tmpPlugin);
        if (typeof fn === "function") {
          fn(api);
        }
      }
      writeCacheIndexFile(hashMap);
    };
  },
  () => {
    return {};
  }
);

export { tailwindcss as default };
